from flask import Flask, render_template, request, session, redirect, url_for
import datetime
from sqlalchemy import desc
from models.database import db_session
from models.models import Schedule, User
from twitter_api import get_twitter_access_token, get_twitter_request_token
import random,string

app = Flask(__name__)

SECRET_KEY = "0pvje8wkFFF6u9nhgat3jm201"
SALT = "hdka56fjwlruiow999qe3e2"
app.secret_key = SECRET_KEY

siteaddress = "https://maruboard.com/"
#siteaddress = "http://127.0.0.1:5000/"

@app.route('/')
@app.route('/home')
def home():
    today = datetime.today().strftime('%Y-%m-%d')
    todayto7day = (datetime.today() + datetime.timedelta(days = 7)).strftime('%Y-%m-%d')
    schedules = Schedule.query.order_by( Schedule.starttime )\
        .filter(Schedule.startdate >= today, Schedule.startdate <= todayto7day, Schedule.canceldate == None).all()

    return render_template(
        'index.html',
        title='まる募 - Twitter連動マルチプレー募集掲示板 -',
        year=datetime.now().year,
        schedules = schedules,
    )

@app.route('/manual')
def manual():
    return render_template(
        'manual.html',
        title='まる募 - 使い方 -',
    )
@app.route('/editSchedule', methods=['GET', 'POST'])
def editSchedule():
    if "session_id" not in session:
        return render_template(
            'error.html',
            message = 'ログインしていないようです。一度トップに戻ってからやり直してください。',
    )
    # GETの時は登録画面へ
    if request.method == 'GET':
        # 編集時指定されたidのページデータを呼び出し
        pid = request.args.get('pageid')
        record = Schedule.query.filter(Schedule.id == pid, Schedule.startdate >= datetime.today().strftime('%Y-%m-%d')).one_or_none()
        if(pid != None):
            if(record == None or session["user_id"] != record.userID):
                #呼び出されたページは本人のデータor本日以降のデータかチェック
                return render_template(
                'error.html',
                message = '不正な操作です。一度トップに戻ってからやり直してください。',
                )
            edittype = "edit"
        else:
            # 新規登録時
            record = Schedule('', datetime.now().day, datetime.now().hour, None, None, '', None, '', '', None, None, None)
            edittype = "new"
        
        return render_template(
            'editSchedule.html',
            title = 'まる募 - スケジュール編集',
            pagetitle = pagestatus(edittype),
            inputData = record,
            TwitterID = session["screen_name"],
            sessionID = session["session_id"],
            edittype = edittype,
            editid = pid
        )
    # POST
    errormessage = []
    edittype = request.form.get('editType')
    sessionid = request.form.get('sessionID')
    if(session["session_id"] != sessionid):
        #セッションIDの確認(CSRF対策)
        return render_template(
        'error.html',
        message = '不正な操作です。一度トップに戻ってからやり直してください。',
        )
    # 新規登録(add)
    if(edittype == "new"):
        record = Schedule(
            request.form.get('title'),
            request.form.get('startdate'),
            request.form.get('starttime'),
            request.form.get('endtime'),
            request.form.get('pratform'), 
            request.form.get('keyword'), 
            session["user_id"], 
            request.form.get('comment'), 
            request.form.get('URL'), 
            None, 
            datetime.now(),
            None
        )
        # 入力チェック
        errormessage = newDataCheck(record)
        # INSERT
        db_session.add(record)

    # 編集(update)
    elif(edittype == "edit"):
        editid = request.form.get('editID')

        record = Schedule.query.filter(Schedule.id == editid).one()

        record.title = request.form.get('title')
        record.startdate = request.form.get('startdate')
        record.starttime =  request.form.get('starttime')
        record.endtime = request.form.get('endtime')
        record.pratform = request.form.get('pratform')
        record.keyword = request.form.get('keyword')
        record.comment =  request.form.get('comment')
        record.userUrl  =  request.form.get('URL')
        # 中止
        if(request.form.get('schedulestatus')=="cancel"):
            record.canceldate = datetime.now()
        elif(request.form.get('schedulestatus')=="held"):
            record.canceldate = None
        #入力チェック
        errormessage = validationCheck(record)

    # エラーがあるときは画面に戻る
    if errormessage != []:
        return render_template(
            'editSchedule.html',
            title = 'まる募 - スケジュール編集',
            pagetitle = pagestatus(edittype),
            inputData = record,
            messages = errormessage,
            TwitterID = session["screen_name"],
            sessionID = session["session_id"],
            edittype = edittype
        )
    
    # ユーザー情報確認
    dbuser = User.query.filter(User.user_id == session["user_id"]).one_or_none()
    #ユーザーがいなければUserレコードも追加
    userid = session["user_id"]
    twiterid = session["screen_name"]
    if(None == dbuser):
        userrecord = User(userid, twiterid)
        db_session.add(userrecord)
    elif(twiterid != dbuser.user_name):
        #TwitterのIDが更新されていたら更新する
        dbuser.user_name = twiterid
    
    #データ確定
    db_session.commit()
    
    # ユーザー画面に遷移
    return redirect(url_for('userpage'))
    
def validationCheck(record):
    messages = []
    # 必須入力チェック
    if(record.title == None):
        messages.append('タイトルが入力されていません') 
    elif(record.startdate == None):
        messages.append('日付が選択されていません')
    elif(record.starttime == None):
        messages.append('開始時刻が選択されていません')
    elif(record.pratform == "default"):
        messages.append('機種が選択されていません')
    # 入力チェック
    if(len(record.title) > 20):
        messages.append('タイトルが20文字を超えています')
    if(len(record.keyword) > 10):
        messages.append('キーワードが10文字を超えています')
    if(len(record.comment) > 300):
        messages.append('コメントが300文字を超えています')
    if((record.userUrl != "") and (not(record.userUrl.startswith("https://twitter.com/")))):
        messages.append('URLがhttps://twitter.com/~で始まっていません')

    return messages

def newDataCheck(record):
    messages = []
    messages = validationCheck(record)

    recentPost = Schedule.query.order_by(desc(Schedule.registdate)).first()
    if (recentPost != None and\
        record.title == recentPost.title and\
        record.startdate == recentPost.startdate and\
        record.starttime == recentPost.starttime and\
        record.endtime == recentPost.endtime and\
        record.pratform == recentPost.pratform and\
        record.keyword == recentPost.keyword and\
        record.comment == recentPost.comment and\
        record.userID == recentPost.userID and\
        record.userUrl == recentPost.userUrl
        ):
        messages.append('投稿内容が重複しています')
    return messages

def pagestatus(edittype):
    if(edittype == "new"):
        pagetitle = '新規登録'
    elif(edittype == "edit"):
        pagetitle = "編集"
    return pagetitle

@app.route('/detail/<int:page_id>', methods=['GET'])
def detail(page_id):
    pageContents = Schedule.query.filter(Schedule.id == page_id).one()
    #UserレコードからTwitterIDを取得
    #twitterid = User.query.filter(User.user_id == pageContents.userID).one()
    
    #URL生成
    shareurl = siteaddress + "detail/" + str(page_id)

    return render_template(
        'detail.html',
        title = 'まる募 - ' + str(pageContents.startdate) + ' ' + str(pageContents.starttime) + 'から' + pageContents.title + 'のマルチプレーを開催予定',
        pageContents = pageContents,
        dataurl = shareurl
        )

@app.route('/search', methods=['GET', 'POST'])
def search():
    if request.method == 'GET':
        date = request.args.get('date')
        title = request.args.get('title')
        pratform = request.args.get('pratform')
        
        if(date == None):
            # 日付リクエストがない時は今日の日付以降をすべて表示
            date = datetime.today().strftime('%Y-%m-%d')
            schedules = Schedule.query.order_by( Schedule.startdate, Schedule.starttime )\
                .filter(Schedule.startdate >= date, Schedule.canceldate == None).all()
        else:
            # あるときはその日付のみを取得
            schedules = Schedule.query.order_by( Schedule.starttime )\
                .filter(Schedule.startdate == date, Schedule.canceldate == None).all()
        if(title == None):
            title = ""
    else:
        date = request.form['date']
        title = request.form['title']
        pratform = request.form['pratform']

        if(pratform =='All'):
            if(title != ""):
                schedules = Schedule.query.order_by( Schedule.starttime )\
                    .filter(Schedule.title == title, Schedule.startdate == date, Schedule.canceldate == None).all()
            else:
                schedules = Schedule.query.order_by( Schedule.starttime )\
                    .filter(Schedule.startdate == date, Schedule.canceldate == None).all()
        else:
            if(title != ""):
                schedules = Schedule.query.order_by( Schedule.starttime )\
                    .filter(Schedule.title == title, Schedule.startdate == date, Schedule.pratform == pratform, Schedule.canceldate == None).all()
            else:
                schedules = Schedule.query.order_by( Schedule.starttime )\
                    .filter(Schedule.startdate == date, Schedule.pratform == pratform, Schedule.canceldate == None).all()

    return render_template(
        'search.html',
        title = 'まる募 - 検索',
        listtitle = '開催予定一覧',
        SearchDate = date,
        SearchTitle = title,
        SearchPratform = pratform,
        schedules = schedules
    )
@app.route('/newSchedule', methods=['GET'])
def newschedule():
    time = request.args.get('time')
    # 最新の登録を取得
    schedules = Schedule.query.order_by( Schedule.startdate, Schedule.starttime )\
        .filter(Schedule.registdate > time, Schedule.canceldate == None).all()
    return render_template(
        'schedulelist.html',
        title = 'まる募 - 新たに登録されたマルチ予告',
        listtitle = '新たに登録されたマルチ予告',
        schedules = schedules
    )
@app.route('/twitter/access_token', methods=['GET'])
def authpage():
    oauth_token = request.args.get('oauth_token')
    oauth_verifier = request.args.get('oauth_verifier')
    
    access_token = get_twitter_access_token(oauth_token, oauth_verifier)
    if len(access_token) == 0:
        # 認証キャンセル時
        return render_template(
            'error.html',
            message = 'ログインしていないようです。一度トップに戻ってからやり直してください。',
        )

    #トークン取得成功
    user_id = access_token['user_id']
    screen_name = access_token['screen_name']

    #セッションに追加
    session["user_id"] = user_id
    session["screen_name"] = screen_name
    session["session_id"] = ''.join(random.choices(string.ascii_letters + string.digits, k=16))

    return redirect(url_for('userpage'))

@app.route("/logout")
def logout():
    session.pop("user_id", None)
    session.pop("screen_name", None)
    session.pop("session_id", None)
    status = "logout"
    oauth_callback = request.args.get('oauth_callback')
    return  render_template(
        'userpage.html',
        title = 'まる募 - ログイン',
        status = status,
        authenticate_endpoint = get_twitter_request_token(oauth_callback)
        )

@app.route('/userpage', methods=['GET'])
def userpage():
    #ログイン確認
    if "user_id" in session and session['user_id']:
        status = "login"
        #DBから同IDの登録データを取得
        todaydate = datetime.today().strftime('%Y-%m-%d')
        pageContents = Schedule.query.filter(Schedule.userID == session['user_id'], todaydate <= Schedule.startdate).limit(20)
        pageContents_old = Schedule.query.filter(Schedule.userID == session['user_id'], todaydate > Schedule.startdate).limit(20)
        #ログイン者用ページに送る
        return render_template(
                'userpage.html',
                title = 'まる募 - ユーザーページ',
                schedules = pageContents,
                schedules_old = pageContents_old,
                status = status,
                login_twitterid = session['screen_name']
        ) 
    else:
        oauth_callback = request.args.get('oauth_callback')
        # 連携画面を呼び出す画面に生成したURLへのリンクを渡す
        return render_template(
            'userpage.html',
            title = 'まる募 - ログイン',
            authenticate_endpoint = get_twitter_request_token(oauth_callback)
            ) 

@app.teardown_appcontext
def session_clear(exception):
    if exception and db_session.is_active:
        db_session.rollback()
    else:
        #session.commit()
        pass
    db_session.close()

if __name__ == "__main__":
    app.run(debug=True)